@@ -31,11 +31,11 @@
 #'@note Column names for the results are created as follows:
 #' Raster data - the column names given in \code{attr_name_rast} are used. The user should
 #' take care to use unique, clear names. For \code{stat_rast} = 'percentage' or 'area', 
-#' the output column name will be concatenated 'p' or 'a', repectively.
+#' the output column name will be concatenated 'p' or 'a', respectively.
 #' For vector data, column names are constructed from the entries in in the column 
 #' \code{attr_name_vect}. For counts of points, the new column name containing the counts
 #' is just the given name. For polygon data ('percentage' or 'area'), the names are constructed using
-#' the unique entries of the column with a concatenated 'p' or 'a', repectively. If, for instance, 
+#' the unique entries of the column with a concatenated 'p' or 'a', respectively. If, for instance, 
 #' for a landuse vector containing the classes 'urban' and 'arable' percentages would be calculated,
 #' edges would contain two new columns 'urbanp' and 'arablep'.
 #'
@@ -76,15 +76,21 @@
 #' 
 #' @examples
 #' \donttest{
+#' # Initiate and setup GRASS
+#' dem_path <- system.file("extdata", "nc", "elev_ned_30m.tif", package = "openSTARS")
 #' if(.Platform$OS.type == "windows"){
-#'   gisbase = "c:/Program Files/GRASS GIS 7.6"
+#'   grass_program_path = "c:/Program Files/GRASS GIS 7.6"
 #'   } else {
-#'   gisbase = "/usr/lib/grass74/"
+#'   grass_program_path = "/usr/lib/grass78/"
 #'   }
-#' initGRASS(gisBase = gisbase,
-#'     home = tempdir(),
-#'     override = TRUE)
 #' 
+#' setup_grass_environment(dem = dem_path, 
+#'                         gisBase = grass_program_path,      
+#'                         remove_GISRC = TRUE,
+#'                         override = TRUE
+#'                         )
+#' gmeta()
+#'                         
 #' # Load files into GRASS
 #' dem_path <- system.file("extdata", "nc", "elev_ned_30m.tif", package = "openSTARS")
 #' sites_path <- system.file("extdata", "nc", "sites_nc.shp", package = "openSTARS")
@@ -92,11 +98,8 @@
 #' preds_v_path <- system.file("extdata", "nc", "pointsources.shp", package = "openSTARS")
 #' preds_r_path <- system.file("extdata", "nc", "landuse_r.tif", package = "openSTARS")
 #'                  
-#' 
-#' setup_grass_environment(dem = dem_path)
 #' import_data(dem = dem_path, sites = sites_path, streams = streams_path,
 #'             predictor_vector = preds_v_path, predictor_raster = preds_r_path)
-#' gmeta()
 #' 
 #' # Derive streams from DEM
 #' # burn in 'streams' 10 meters
@@ -115,8 +118,7 @@
 #' parameters = list(
 #'   elevation = "dem",
 #'     slope = "slope"
-#'     ))
-#' 
+#'     )) 
 #' 
 #' # Prepare edges
 #' calc_edges()
@@ -129,9 +131,9 @@
 #' # Plot eges with percentage of forest in the catchment (lusep_5) as line width
 #' edges <- readVECT('edges', ignore.stderr = TRUE)
 #' head(edges@data)
-#' lu <- readRAST("landuse_r", ignore.stderr = TRUE)
+#' lu <- readRAST("landuse_r", ignore.stderr = TRUE, plugin = FALSE)
 #' 
-#'  # plot landuse data
+#' # plot landuse data
 #' library(raster)
 #' op <- par()
 #' par(xpd = FALSE)
@@ -143,6 +145,7 @@
 #'   legend = c("developed", "agriculture", "herbaceous", "shrubland", "forest", "water", "sediment"),
 #'   fill = c("red", "goldenrod", "green", "forestgreen","darkgreen", "blue", "lightblue"),
 #'   horiz = TRUE, inset = -0.175)
+#'   # line width is relative to the area of land use class 5 (forest) in the rca of the edge segment
 #' plot(edges, lwd = edges$lusep_5_c * 10, add = TRUE)    
 #' par <- op
 #' }
@@ -248,7 +251,7 @@ calc_attributes_edges <- function(input_raster = NULL, stat_rast = NULL, attr_na
     } else {
       mes <- input_vector
     }
-    stop(paste0("Missing input vector data ", paste0("'",mes,"'", collapse = ", "), 
+    stop(paste0("Missing input vector data ", paste0("'",mes ,"'", collapse = ", "), 
                 ". Please give valid vector file names. \nAvailable vector files are ",  
                 paste0("'",vect,"'", collapse = ", ")))
   }
@@ -443,17 +446,25 @@ calc_attributes_edges <- function(input_raster = NULL, stat_rast = NULL, attr_na
                     operator = "and",
                     output = "temp_inters"
                   ), ignore.stderr = T)
-        execGRASS("v.db.addcolumn", flags = "quiet",
-                  parameters = list(
-                    map =  "temp_inters",
-                    columns = "area double"
-                  ), ignore.stderr = TRUE)
-        execGRASS("v.to.db", flags = c("quiet"),
-                  parameters = list(
-                    map =  "temp_inters",
-                    option = "area",
-                    columns = "area"
-                  ), ignore.stderr = TRUE)
+
+        ## GRASS version below 7.8
+        ## v.to.db needs the column to be pobulated to exist; from 7.8 onward this column is created and existing ones are not automatically overwritten   
+        # if(compareVersion(strsplit(system2("grass",  "--version", stdout = TRUE, stderr = TRUE)[1], " ")[[1]][3], "7.8") < 0){
+        #   execGRASS("v.db.addcolumn", flags = "quiet",
+        #           parameters = list(
+        #             map =  "temp_inters",
+        #             columns = "area double"
+        #           ), ignore.stderr = TRUE)
+        # }
+        # execGRASS("v.to.db", flags = c("quiet"),
+        #           parameters = list(
+        #             map =  "temp_inters",
+        #             option = "area",
+        #             columns = "area"
+        #           ), ignore.stderr = TRUE)
+        
+        grass_v.to.db(map = "temp_inters", option = "area", columns = "area", format = "double precision")
+        
         cname <- paste0("b_", attr_name_vect[j])
         dt.dat <- do.call(rbind,strsplit(
           execGRASS("db.select",
@@ -469,6 +480,14 @@ calc_attributes_edges <- function(input_raster = NULL, stat_rast = NULL, attr_na
         dt.dat <- dt.dat[, .(area = sum(area)), c("stream", cname)] 
         dt.dat <- dcast(dt.dat, paste0("stream  ~ b_", attr_name_vect[j]), value.var = "area")
         #setnames(dt.dat, names(dt.dat)[-1], paste0("p", names(dt.dat)[-1]))
+        
+        # remove temporary files
+        execGRASS("g.remove",
+                  flags = c("quiet", "f"),
+                  parameters = list(
+                    type = "vector",
+                    name = "temp_inters"
+                  ), ignore.stderr = TRUE)
       } else { # if point data
         execGRASS("v.vect.stats", flags = "quiet", 
                   parameters = list(
@@ -533,14 +552,6 @@ calc_attributes_edges <- function(input_raster = NULL, stat_rast = NULL, attr_na
                 other_table = "edge_attributes",
                 other_column = "stream"
               ))
-    
-    # remove temporary files
-    execGRASS("g.remove",
-              flags = c("quiet", "f"),
-              parameters = list(
-                type = "vector",
-                name = "temp_inters"
-              ), ignore.stderr = TRUE)
   }
   
   cnames_edges2 <- execGRASS("db.columns", flags = "quiet",
@@ -725,7 +736,7 @@ calc_catchment_attributes_vect_rec <- function(dt, id, stat_vect, attr_name_vect
 
 
 #' get_n_val_raster
-#' Returns the number of differnt values in the raster.
+#' Returns the number of different values in the raster.
 #'
 #' @description Returns the number of different values in the input raster.
 #'
@@ -743,7 +754,7 @@ get_n_val_raster <- function(raster_name){
 }
 
 #' get_all_raster_values
-#' Returns all unuque values in the raster
+#' Returns all unique values in the raster
 #' 
 #' @description Returns the number of different values in the input raster.
 #'
